@@ -10,16 +10,12 @@ use RDF::Trine qw[iri literal blank variable statement];
 use RDF::Generator::Void::Stats;
 # use less ();
 use utf8;
-use URI::Split qw(uri_split uri_join);
-
-use aliased 'RDF::Generator::Void::Meta::Attribute::ObjectList';
 
 # Define some namespace prefixes
 my $void = RDF::Trine::Namespace->new('http://rdfs.org/ns/void#');
 my $rdf  = RDF::Trine::Namespace->new('http://www.w3.org/1999/02/22-rdf-syntax-ns#');
 my $xsd  = RDF::Trine::Namespace->new('http://www.w3.org/2001/XMLSchema#');
 my $dct  = RDF::Trine::Namespace->new('http://purl.org/dc/terms/');
-my $prov = RDF::Trine::Namespace->new('http://www.w3.org/ns/prov#');
 
 =head1 NAME
 
@@ -27,11 +23,16 @@ RDF::Generator::Void - Generate VoID descriptions based on data in an RDF model
 
 =head1 VERSION
 
-Version 0.15_2
+Version 0.02
+
+Note that this is a beta release. It has the core functionality in
+place to create a basic VoID description and what's there should be
+working well. Nevertheless significant changes in this module may be
+coming up really soon.
 
 =cut
 
-our $VERSION = '0.15_2';
+our $VERSION = '0.02';
 
 =head1 SYNOPSIS
 
@@ -55,7 +56,7 @@ For a description of VoID, see L<http://www.w3.org/TR/void/>.
 
 =head1 METHODS
 
-=head2 new(inmodel => $mymodel, dataset_uri => URI->new($dataset_uri), level => 1);
+=head2 new(inmodel => $mymodel, dataset_uri => URI->new($dataset_uri));
 
 The constructor. It can be called with two parameters, namely,
 C<inmodel> which is a model we want to describe and C<dataset_uri>,
@@ -115,7 +116,7 @@ values. Methods starting with C<add_> takes a list of values to add,
 and those starting with C<has_no_> return a boolean value, false if
 the array is empty.
 
-=head3 C<all_vocabularies>, C<add_vocabularies>, C<has_no_vocabularies>
+=head3 C<vocabulary>, C<all_vocabularies>, C<add_vocabularies>, C<has_no_vocabularies>
 
 Methods to manipulate a list of vocabularies used in the dataset. The
 values should be a string that represents the URI of a vocabulary.
@@ -125,9 +126,19 @@ values should be a string that represents the URI of a vocabulary.
 # All the following attributes have that in common that they
 # automatically the method names also specified in handles, to
 # manipulate and query the data.
-has _vocabularies => ( traits => [ObjectList] );
+has vocabulary => (
+						 is       => 'rw',
+						 traits   => ['Array'],
+						 isa      => 'ArrayRef[Str]',
+						 default  => sub { [] },
+						 handles  => {
+										  all_vocabularies    => 'uniq',
+										  add_vocabularies    => 'push',
+										  has_no_vocabularies => 'is_empty',
+										 },
+						);
 
-=head3 C<all_endpoints>, C<add_endpoints>, C<has_no_endpoints>
+=head3 C<endpoint>, C<all_endpoints>, C<add_endpoints>, C<has_no_endpoints>
 
 Methods to manipulate a list of SPARQL endpoints that can be used to
 query the dataset. The values should be a string that represents the
@@ -136,9 +147,19 @@ URI of a SPARQL endpoint.
 =cut
 
 
-has _endpoints => ( traits => [ObjectList] );
+has endpoint => (
+					  is       => 'rw',
+					  traits   => ['Array'],
+					  isa      => 'ArrayRef[Str]',
+					  default  => sub { [] },
+					  handles  => {
+										all_endpoints    => 'uniq',
+										add_endpoints    => 'push',
+										has_no_endpoints => 'is_empty',
+									  },
+					 );
 
-=head3 C<all_titles>, C<add_titles>, C<has_no_titles>
+=head3 C<title>, C<all_titles>, C<add_titles>, C<has_no_titles>
 
 Methods to manipulate the titles of the datasets. The values should be
 L<RDF::Trine::Node::Literal> objects, and should be set with
@@ -147,13 +168,20 @@ language. Typically, you would have a value per language.
 =cut
 
 
-has _titles => ( 
-				  traits => [ObjectList],
+has title => (
+				  is       => 'rw',
+				  traits   => ['Array'],
 				  isa      => 'ArrayRef[RDF::Trine::Node::Literal]',
+				  default  => sub { [] },
+				  handles  => {
+									all_titles    => 'uniq',
+									add_titles    => 'push',
+									has_no_titles => 'is_empty',
+								  },
 				 );
 
 
-=head3 C<all_licenses>, C<add_licenses>, C<has_no_licenses>
+=head3 C<license>, C<all_licenses>, C<add_licenses>, C<has_no_licenses>
 
 Methods to manipulate a list of licenses that regulates the use of the
 dataset. The values should be a string that represents the URI of a
@@ -161,7 +189,17 @@ license.
 
 =cut
 
-has _licenses => ( traits => [ObjectList] );
+has license => (
+					 is       => 'rw',
+					 traits   => ['Array'],
+					 isa      => 'ArrayRef[Str]',
+					 default  => sub { [] },
+					 handles  => {
+									  all_licenses    => 'uniq',
+									  add_licenses    => 'push',
+									  has_no_licenses => 'is_empty',
+									 },
+					);
 
 =head3 C<urispace>, C<has_urispace>
 
@@ -182,18 +220,6 @@ has urispace => (
 
 =head2 Running this stuff
 
-=head3 C<level>, C<has_level>
-
-Set the level of detail. 0 doesn't do any statistics or heuristics, 1
-has some statistics for the dataset as a whole, 2 will give some
-partition statistics and 3 will give subject and object counts for
-property partitions. Setting no level will give everything.
-
-=cut
-
-has level => (is => 'rw', isa => 'Int', predicate => 'has_level');
-
-
 =head3 C<stats>, C<clear_stats>, C<has_stats>
 
 Method to compute a statistical summary for the data in the dataset,
@@ -219,20 +245,18 @@ sub _build_stats {
 }
 
 
-=head3 generate( [ $model ] )
+=head3 generate
 
-Returns the VoID as an RDF::Trine::Model. You may pass a model with
-statements as argument to this method. This model may then contain
-arbitrary RDF that will be added to the RDF model. If you do not send
-a model, one will be created for you.
+Returns the VoID as an RDF::Trine::Model.
 
 =cut
 
 sub generate {
 	my $self = shift;
-	my $void_model = shift || RDF::Trine::Model->temporary_model;
 
-	local $self->{void_model} = $void_model;
+	# Create a model for adding VoID description
+	local $self->{void_model} =
+	  my $void_model = RDF::Trine::Model->temporary_model;
 
 	# Start generating the actual VoID statements
 	$void_model->add_statement(statement(
@@ -241,26 +265,19 @@ sub generate {
 													 $void->Dataset,
 													));
 
-	my ($scheme, $auth, $path, $query, $frag) = uri_split($self->dataset_uri->uri_value);
-	if ($frag) { # Then, we have a document that could be described with provenance
-		my $uri = iri(uri_join($scheme, $auth, $path, $query, undef));
-		my $blank = blank();
-		$void_model->add_statement(statement($uri,
-														 $prov->wasGeneratedBy,
-														 $blank));
-		(my $ver = $VERSION) =~ s/\./-/;
-		my $release_uri = iri("http://purl.org/NET/cpan-uri/dist/RDF-Generator-Void/v_$ver");
-		$void_model->add_statement(statement($blank,
-														 $prov->wasAssociatedWith,
-														 $release_uri));
-		$void_model->add_statement(statement($release_uri,
-														 $rdf->type,
-													    $prov->SoftwareAgent));
-		$void_model->add_statement(statement($release_uri,
-														 iri('http://www.w3.org/2000/01/rdf-schema#label'),
-													    literal("RDF::Generator::Void, Version $VERSION", 'en')));
+	if ($self->has_urispace) {
+		$void_model->add_statement(statement(
+														 $self->dataset_uri,
+														 $void->uriSpace,
+														 literal($self->urispace)
+														));
+		$self->_generate_counts($void->entities, $self->stats->entities);
 	}
 
+	$self->_generate_counts($void->distinctSubjects, $self->stats->subjects);
+	$self->_generate_counts($void->properties, $self->stats->properties);
+	$self->_generate_counts($void->distinctObjects, $self->stats->objects);
+
 
 	foreach my $endpoint ($self->all_endpoints) {
 		$void_model->add_statement(statement(
@@ -292,28 +309,8 @@ sub generate {
 													 $void->triples,
 													 literal($self->inmodel->size, undef, $xsd->integer),
 													));
-
-	if ($self->has_urispace) {
-		$void_model->add_statement(statement(
-														 $self->dataset_uri,
-														 $void->uriSpace,
-														 literal($self->urispace)
-														));
-		return $void_model if ($self->has_level && ($self->level == 0));
-		$self->_generate_counts($void->entities, $self->stats->entities);
-	}
-
-	return $void_model if ($self->has_level && $self->level == 0);
-	$self->_generate_counts($void->distinctSubjects, $self->stats->subjects);
-	$self->_generate_counts($void->properties, $self->stats->properties);
-	$self->_generate_counts($void->distinctObjects, $self->stats->objects);
-
 	$self->_generate_most_common_vocabs($self->stats) if $self->has_stats;
-
-	return $void_model if ($self->has_level && $self->level <= 1);
-
-	$self->_generate_propertypartitions;
-	$self->_generate_classpartitions;
+  
 	return $void_model;
 }
 
@@ -327,56 +324,6 @@ sub _generate_counts {
 															  ));
 }
 
-sub _generate_propertypartitions {
-  my ($self) = @_;
-  return undef unless $self->has_stats;
-  my $properties = $self->stats->propertyPartitions;
-  while (my ($uri, $counts) = each(%{$properties})) {
-    my $blank = blank();
-    $self->{void_model}->add_statement(statement(
-						 $self->dataset_uri,
-						 $void->propertyPartition,
-						 $blank));
-    $self->{void_model}->add_statement(statement($blank,
-						 $void->property,
-						 iri($uri)));
-    $self->{void_model}->add_statement(statement($blank,
-						 $void->triples,
-						 literal($counts->{'triples'}, undef, $xsd->integer)));
-	 # OK, so sometimes, one has to balance elegance and performance...
-	 if ($counts->{'countsubjects'}) {
-		 $self->{void_model}->add_statement(statement($blank,
-						 $void->distinctSubjects,
-						 literal(scalar keys %{$counts->{'countsubjects'}}, undef, $xsd->integer)));
-		 $self->{void_model}->add_statement(statement($blank,
-						 $void->distinctObjects,
-						 literal(scalar keys %{$counts->{'countobjects'}}, undef, $xsd->integer)));
-	 }
-
-		 
-
-  }
-}
-
-sub _generate_classpartitions {
-  my ($self) = @_;
-  return undef unless $self->has_stats;
-  my $classes = $self->stats->classPartitions;
-  while (my ($uri, $count) = each(%{$classes})) {
-    my $blank = blank();
-    $self->{void_model}->add_statement(statement(
-						 $self->dataset_uri,
-						 $void->classPartition,
-						 $blank));
-    $self->{void_model}->add_statement(statement($blank,
-						 $void->class,
-						 iri($uri)));
-    $self->{void_model}->add_statement(statement($blank,
-						 $void->triples,
-						 literal($count, undef, $xsd->integer)));
-  }
-}
-
 sub _generate_most_common_vocabs {
 	my ($self) = @_;
 
@@ -401,13 +348,20 @@ sub _generate_most_common_vocabs {
 
 Kjetil Kjernsmo C<< <kjetilk@cpan.org> >>
 Toby Inkster C<< <tobyink@cpan.org> >>
+Tope Omitola, C<< <tope.omitola at googlemail.com> >>
 
 =head1 TODO
 
 =over
 
+=item * Allow arbitrary RDF to be added to the VoID.
+
+=item * Larger test dataset for more extensive tests.
+
 =item * URI regexps support.
 
+=item * Partitioning based on properties and classes.
+
 =item * Technical features (esp. serializations).
 
 =item * Example resources and root resources.
@@ -424,11 +378,9 @@ Toby Inkster C<< <tobyink@cpan.org> >>
 
 =item * Set URI space on partitions.
 
-=item * Use L<CHI> to cache?
-
-=item * Use schema introspection to generate property attributes with L<MooseX::Semantics>.
-
+=item * Conditional updates based on model ETags.
 
+=item * Save the description to files?
 
 =back
 
@@ -437,13 +389,6 @@ Toby Inkster C<< <tobyink@cpan.org> >>
 
 Please report any bugs you find to L<https://github.com/kjetilk/RDF-Generator-Void/issues>
 
-Note that any claim that this module will generate a void in
-spacetime, a wormhole, black hole, or funny philosophy is totally
-bogus and without any scientific merit whatsoever. The lead author has
-made elaborate precautions to avoid any such issues, and expects
-everyone to take his word for it. Oh, BTW, should it just happen
-anyway, it won't L<hurt much|http://news.sciencemag.org/sciencenow/2012/03/scienceshot-one-black-hole-wont-.html>.
-
 
 =head1 SUPPORT
 
@@ -475,12 +420,10 @@ L<https://metacpan.org/module/RDF::Generator::Void>
 
 =head1 ACKNOWLEDGEMENTS
 
-Many thanks to Konstantin Baierer for help with L<RDF::Generator::Void::Meta::Attribute::ObjectList>.
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2012 Toby Inkster.
-Copyright 2012,2013,2016 Kjetil Kjernsmo.
+Copyright 2012 Tope Omitola, Kjetil Kjernsmo, Toby Inkster.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of either: the GNU General Public License as published
